/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2015 Bhushan Shah <bshah@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "debug.h"

Q_LOGGING_CATEGORY(PLASMAPA, "org.kde.plasma.pulseaudio", QtWarningMsg)
