/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2015 Bhushan Shah <bshah@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef DEBUG_H
#define DEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(PLASMAPA)

#endif
